/* Copyright (C) 2008, Cyril Hu (cyrilhu@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#include <stddef.h>
#include <stdio.h>

/* Invoke fopen() and return rewind()ed fp on success,
 * return NULL on open failure or empty file. */
FILE *feopen(const char *file, const char *mode)
{
	long offset;
	FILE *fp = fopen(file, mode);
#ifdef IO_DEBUG
	printf("feopen(): opening %s in %s mode!\n", file, mode);
#endif
	if(!fp) {
#ifdef IO_DEBUG
		printf("feopen(): fopen() failed !\n");
#endif
		return NULL;
	}
	fseek(fp, 0, SEEK_END);
	offset = ftell(fp);
#ifdef IO_DEBUG
	printf("feopen(): file offset=%ld\n", offset);
#endif
	if(offset <= 0) {
		fclose(fp);
		return NULL;
	}
	fseek(fp, 0, SEEK_SET);
	return fp;
}

#ifdef IO_FFOPEN_DEBUG
int main(int argc, char *argv[])
{
	FILE *fp;
       	fp = feopen(argv[1], "r");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[2], "r+");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[3], "rb");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[4], "rb+");
	printf("fp=%p\n", fp);
#if 1
       	fp = feopen(argv[5], "a");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[6], "a+");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[7], "ab");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[8], "ab+");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[9], "w");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[10], "w+");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[11], "wb");
	printf("fp=%p\n", fp);
       	fp = feopen(argv[12], "wb+");
	printf("fp=%p\n", fp);
#endif
	return 0;
}
#endif
