/* Copyright (C) 2008, Cyril Hu (cyrilhu@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

/* stdlib.h extension.
 */
#ifndef _ULLIB_H
#define _ULLIB_H

#include <stddef.h>
#include "ulbool.h"
#include "uldef.h"
_BEGIN_DECLS

/* Invoke putchar('\a'). */
void beep(void);

/* Various swap unit, (c=char, i=int, u=unsigned, s=size_t, ss=ssize_t...). */
void cswap	(void *a, void *b);
void hswap	(void *a, void *b);
void iswap	(void *a, void *b);
void lswap	(void *a, void *b);
#if __STDC_VERSION__ >= 199901L
void llswap	(void *a, void *b);
#endif
void sswap	(void *a, void *b);

/* Various compare unit for function like qsort(), 
 * (c=char, i=int, s=size_t, ss=ssize_t... a=asc, d=des). */
int ccmpa	(const void *a, const void *b);
int ccmpd	(const void *a, const void *b);
int uccmpa	(const void *a, const void *b);
int uccmpd	(const void *a, const void *b);
int hcmpa	(const void *a, const void *b);
int hcmpd	(const void *a, const void *b);
int uhcmpa	(const void *a, const void *b);
int uhcmpd	(const void *a, const void *b);
int icmpa	(const void *a, const void *b);
int icmpd	(const void *a, const void *b);
int uicmpa	(const void *a, const void *b);
int uicmpd	(const void *a, const void *b);
int lcmpa	(const void *a, const void *b);
int lcmpd	(const void *a, const void *b);
int ulcmpa	(const void *a, const void *b);
int ulcmpd	(const void *a, const void *b);
#if __STDC_VERSION__ >= 199901L
int llcmpa	(const void *a, const void *b);
int llcmpd	(const void *a, const void *b);
int ullcmpa	(const void *a, const void *b);
int ullcmpd	(const void *a, const void *b);
#endif
int scmpa	(const void *a, const void *b);
int scmpd	(const void *a, const void *b);

/* To see whether array is keyword ordered or not,
 * return true if ordered, false if not. */
bool array_issorted(const void *src, size_t n, size_t sz, int (*cmp)(const void *, const void *));

/* Find array's min value or max value element,
 * return a pointer to this element. */
void *array_getmaxormin(const void *src, size_t n, size_t sz, int (*cmp)(const void *, const void *));

_END_DECLS
#endif
